/***************************************************************************
 *   Copyright (C) 2003 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qlabel.h>
#include <qpopupmenu.h>
#include <qscrollview.h>
#include <qglobal.h>
#include <qpainter.h>
#include <kdebug.h>
#include <kapplication.h>
#include <kpushbutton.h>
#include <kcombobox.h>
#include <kapplication.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include "kxstitch.h"
#include "layouttemplatedialog.h"
#include "configuration.h"

struct PAPERSIZE {
  enum KPrinter::PageSize pageSize;
  char *name;
  int width;    // mm portrait
  int height;   // mm portrait
} pageSizeList[] =
{
  { KPrinter::A0,         "A0",         841,  1189  },
  { KPrinter::A1,         "A1",         594,  841   },
  { KPrinter::A2,         "A2",         420,  594   },
  { KPrinter::A3,         "A3",         297,  420   },
  { KPrinter::A4,         "A4",         210,  297   },
  { KPrinter::A5,         "A5",         148,  210   },
  { KPrinter::A6,         "A6",         105,  148   },
  { KPrinter::A7,         "A7",         74,   105   },
  { KPrinter::A8,         "A8",         52,   74    },
  { KPrinter::A9,         "A9",         37,   52    },
  { KPrinter::B0,         "B0",         1030, 1456  },
  { KPrinter::B1,         "B1",         728,  1030  },
  { KPrinter::B10 ,       "B10",        32,   45    },
  { KPrinter::B2,         "B2",         515,  728   },
  { KPrinter::B3,         "B3",         364,  515   },
  { KPrinter::B4,         "B4",         257,  364   },
  { KPrinter::B5,         "B5",         182,  257   },
  { KPrinter::B6,         "B6",         128,  182   },
  { KPrinter::B7,         "B7",         91,   128   },
  { KPrinter::B8,         "B8",         64,   91    },
  { KPrinter::B9,         "B9",         45,   64    },
  { KPrinter::C5E,        "C5E",        163,  229   },
  { KPrinter::Comm10E,    "Comm10E",    105,  241   },
  { KPrinter::DLE,        "DLE",        110,  220   },
  { KPrinter::Executive,  "Executive",  191,  254   },
  { KPrinter::Folio,      "Folio",      210,  330   },
  { KPrinter::Ledger,     "Ledger",     432,  279   },
  { KPrinter::Legal,      "Legal",      216,  356   },
  { KPrinter::Letter,     "Letter",     216,  279   },
  { KPrinter::Tabloid,    "Tabloid",    279,  432   }
};

#define NUMBERPAGESIZES (sizeof(pageSizeList)/sizeof(struct PAPERSIZE))

LayoutTemplatePageElement::LayoutTemplatePageElement(QWidget *parent)
  : QWidget(parent)
{
  kdDebug() << "Entering LayoutTemplatePageElement::LayoutTemplatePageElement" << endl;
  kdDebug() << "Leaving LayoutTemplatePageElement::LayoutTemplatePageElement" << endl;
}

LayoutTemplatePageElement::~LayoutTemplatePageElement()
{
  kdDebug() << "Entering LayoutTemplatePageElement::~LayoutTemplatePageElement" << endl;
  kdDebug() << "Leaving LayoutTemplatePageElement::~LayoutTemplatePageElement" << endl;
}

void LayoutTemplatePageElement::print(QPainter*)
{
  kdDebug() << "Entering LayoutTemplatePageElement::print" << endl;
  kdDebug() << "Leaving LayoutTemplatePageElement::print" << endl;
}

LayoutTemplatePage::LayoutTemplatePage(QWidget *parent)
  : QWidget(parent)
{
  kdDebug() << "Entering LayoutTemplatePage::LayoutTemplatePage" << endl;
  setPaletteBackgroundColor(Qt::white);
  m_marginLeft = Configuration::template_MarginLeft();
  m_marginRight = Configuration::template_MarginRight();
  m_marginTop = Configuration::template_MarginTop();
  m_marginBottom = Configuration::template_MarginBottom();
  m_gridX = Configuration::template_GridX();
  m_gridY = Configuration::template_GridY();
//  m_orientation = Configuration::template_PageOrientation();
//  setPageSize(KXSConfig().Template_PageSize);
  setOrientation(KPrinter::Portrait);
  setPageSize(KPrinter::A0);
  m_showGrid = Configuration::template_ShowGrid();
  kdDebug() << "Leaving LayoutTemplatePage::LayoutTemplatePage" << endl;
}

LayoutTemplatePage::~LayoutTemplatePage()
{
  kdDebug() << "Entering LayoutTemplatePage::~LayoutTemplatePage" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::~LayoutTemplatePage" << endl;
}

void LayoutTemplatePage::setOrientation(KPrinter::Orientation orientation)
{
  kdDebug() << "Entering LayoutTemplatePage::setOrientation" << endl;
  m_orientation = orientation;
  kdDebug() << "Leaving LayoutTemplatePage::setOrientation" << endl;
}

KPrinter::Orientation LayoutTemplatePage::orientation() const
{
  kdDebug() << "Entering LayoutTemplatePage::orientation" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::orientation" << endl;
  return m_orientation;
}

void LayoutTemplatePage::setPageSize(KPrinter::PageSize pageSize)
{
  kdDebug() << "Entering LayoutTemplatePage::setPageSize" << endl;
  m_pageSize = pageSize;
  for (int i = 0 ; i < NUMBERPAGESIZES ; i++)
  {
    if (m_pageSize == pageSizeList[i].pageSize)
    {
      m_width = pageSizeList[i].width;
      m_height = pageSizeList[i].height;
    }
  }
  kdDebug() << "Leaving LayoutTemplatePage::setPageSize" << endl;
}

void LayoutTemplatePage::setPageSize(int i)
{
  kdDebug() << "Entering LayoutTemplatePage::setPageSize(int)" << endl;
  m_pageSize = pageSizeList[i].pageSize;
  m_width = pageSizeList[i].width;
  m_height = pageSizeList[i].height;
  updatePageDisplay();
  kdDebug() << "Leaving LayoutTemplatePage::setPageSize(int)" << endl;
}

KPrinter::PageSize LayoutTemplatePage::pageSize() const
{
  kdDebug() << "Entering LayoutTemplatePage::pageSize" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::pageSize" << endl;
  return m_pageSize;
}

int LayoutTemplatePage::width() const
{
  kdDebug() << "Entering LayoutTemplatePage::width" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::width" << endl;
  return (m_orientation == KPrinter::Portrait)?m_width:m_height;
}

int LayoutTemplatePage::height() const
{
  kdDebug() << "Entering LayoutTemplatePage::height" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::height" << endl;
  return (m_orientation == KPrinter::Portrait)?m_height:m_width;
}

void LayoutTemplatePage::setPageNumber(int page)
{
  kdDebug() << "Entering LayoutTemplatePage::setCurrentPage" << endl;
  m_pageNumber = page;
  kdDebug() << "Leaving LayoutTemplatePage::setCurrentPage" << endl;
}

int LayoutTemplatePage::pageNumber() const
{
  kdDebug() << "Entering LayoutTemplatePage::currentPage" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::currentPage" << endl;
  return m_pageNumber;
}

void LayoutTemplatePage::print(KPrinter *p)
{
  kdDebug() << "Entering LayoutTemplatePage::print" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::print" << endl;
}

void LayoutTemplatePage::paintEvent(QPaintEvent *p)
{
  kdDebug() << "Entering LayoutTemplatePage::paintEvent" << endl;
  updatePageDisplay();
  QPainter *painter = new QPainter(this);
  drawGrid(painter);
  delete painter;
  kdDebug() << "Leaving LayoutTemplatePage::paintEvent" << endl;
}

void LayoutTemplatePage::mousePressEvent(QMouseEvent *e)
{
  kdDebug() << "Entering LayoutTemplatePage::mousePressEvent" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::mousePressEvent" << endl;
}

void LayoutTemplatePage::mouseMoveEvent(QMouseEvent *e)
{
  kdDebug() << "Entering LayoutTemplatePage::mouseMoveEvent" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::mouseMoveEvent" << endl;
}

void LayoutTemplatePage::mouseReleaseEvent(QMouseEvent *e)
{
  kdDebug() << "Entering LayoutTemplatePage::mouseReleaseEvent" << endl;
  kdDebug() << "Leaving LayoutTemplatePage::mouseReleaseEvent" << endl;
}

void LayoutTemplatePage::contextMenuEvent(QContextMenuEvent *e)
{
  kdDebug() << "Entering LayoutTemplatePage::contextMenuEvent" << endl;
  KXStitchApp *app = static_cast<KXStitchApp*>(topLevelWidget()->parentWidget());  // topLevelWidget() returns a pointer to the dialog
  KActionCollection *actionCollection = app->actionCollection();

  KActionMenu *elementMenu = new KActionMenu(i18n("Insert Element"));
  elementMenu->insert(new KAction(i18n("Page Number"), 0, this, SLOT(slotElementPageNumber()), actionCollection, "pagenumber"));
  elementMenu->insert(new KAction(i18n("Pattern Title"), 0, this, SLOT(slotElementTitle()), actionCollection, "title"));
  elementMenu->insert(new KAction(i18n("Fabric"), 0, this, SLOT(slotElementFabric()), actionCollection, "fabric"));
  elementMenu->insert(new KAction(i18n("Fabric Color"), 0, this, SLOT(slotElementFabricColor()), actionCollection, "fabriccolor"));
  elementMenu->insert(new KAction(i18n("Pattern Grid"), 0, this, SLOT(slotElementPageGrid()), actionCollection, "grid"));
  elementMenu->insert(new KAction(i18n("Copyright"), 0, this, SLOT(slotElementCopyright()), actionCollection, "copyright"));
  elementMenu->insert(new KAction(i18n( "Author"), 0, this, SLOT(slotElementAuthor()), actionCollection, "author"));
  KActionMenu *pageSizeMenu = new KActionMenu(i18n("Page Size"));
  pageSizeMenu->insert(new KAction(i18n("A0"), 0, this, SLOT(slotPageA0()), actionCollection, "a0"));
  pageSizeMenu->insert(new KAction(i18n("A1"), 0, this, SLOT(slotPageA1()), actionCollection, "a1"));
  pageSizeMenu->insert(new KAction(i18n("A2"), 0, this, SLOT(slotPageA2()), actionCollection, "a2"));
  pageSizeMenu->insert(new KAction(i18n("A3"), 0, this, SLOT(slotPageA3()), actionCollection, "a3"));
  pageSizeMenu->insert(new KAction(i18n("A4"), 0, this, SLOT(slotPageA4()), actionCollection, "a4"));
  pageSizeMenu->insert(new KAction(i18n("A5"), 0, this, SLOT(slotPageA5()), actionCollection, "a5"));
  pageSizeMenu->insert(new KAction(i18n("A6"), 0, this, SLOT(slotPageA6()), actionCollection, "a6"));
  pageSizeMenu->insert(new KAction(i18n("A7"), 0, this, SLOT(slotPageA7()), actionCollection, "a7"));
  pageSizeMenu->insert(new KAction(i18n("A8"), 0, this, SLOT(slotPageA8()), actionCollection, "a8"));
  pageSizeMenu->insert(new KAction(i18n("A9"), 0, this, SLOT(slotPageA9()), actionCollection, "a9"));
  pageSizeMenu->insert(new KAction(i18n("B0"), 0, this, SLOT(slotPageB0()), actionCollection, "b0"));
  pageSizeMenu->insert(new KAction(i18n("B1"), 0, this, SLOT(slotPageB1()), actionCollection, "b1"));
  pageSizeMenu->insert(new KAction(i18n("B2"), 0, this, SLOT(slotPageB2()), actionCollection, "b2"));
  pageSizeMenu->insert(new KAction(i18n("B3"), 0, this, SLOT(slotPageB3()), actionCollection, "b3"));
  pageSizeMenu->insert(new KAction(i18n("B4"), 0, this, SLOT(slotPageB4()), actionCollection, "b4"));
  pageSizeMenu->insert(new KAction(i18n("B5"), 0, this, SLOT(slotPageB5()), actionCollection, "b5"));
  pageSizeMenu->insert(new KAction(i18n("B6"), 0, this, SLOT(slotPageB6()), actionCollection, "b6"));
  pageSizeMenu->insert(new KAction(i18n("B7"), 0, this, SLOT(slotPageB7()), actionCollection, "b7"));
  pageSizeMenu->insert(new KAction(i18n("B8"), 0, this, SLOT(slotPageB8()), actionCollection, "b8"));
  pageSizeMenu->insert(new KAction(i18n("B9"), 0, this, SLOT(slotPageB9()), actionCollection, "b9"));
  pageSizeMenu->insert(new KAction(i18n("B10"), 0, this, SLOT(slotPageB10()), actionCollection, "b10"));
  pageSizeMenu->insert(new KAction(i18n("Comm10E"), 0, this, SLOT(slotPageComm10E()), actionCollection, "comm10e"));
  pageSizeMenu->insert(new KAction(i18n("DLE"), 0, this, SLOT(slotPageDLE()), actionCollection, "dle"));
  pageSizeMenu->insert(new KAction(i18n("Executive"), 0, this, SLOT(slotPageExecutive()), actionCollection, "executive"));
  pageSizeMenu->insert(new KAction(i18n("Folio"), 0, this, SLOT(slotPageFolio()), actionCollection, "folio"));
  pageSizeMenu->insert(new KAction(i18n("Ledger"), 0, this, SLOT(slotPageLedger()), actionCollection, "ledger"));
  pageSizeMenu->insert(new KAction(i18n("Legal"), 0, this, SLOT(slotPageLegal()), actionCollection, "legal"));
  pageSizeMenu->insert(new KAction(i18n("Letter"), 0, this, SLOT(slotPageLetter()), actionCollection, "letter"));
  pageSizeMenu->insert(new KAction(i18n("Tabloid"), 0, this, SLOT(slotPageTabloid()), actionCollection, "tabloid"));
  KActionMenu *orientationMenu = new KActionMenu(i18n("Orientation"));
  orientationMenu->insert(new KAction(i18n("Portrait"), 0, this, SLOT(slotPagePortrait()), actionCollection, "portrait"));
  orientationMenu->insert(new KAction(i18n("Landscape"), 0, this, SLOT(slotPageLandscape()), actionCollection, "landscape"));
  KPopupMenu *contextPopup = new KPopupMenu(this);
  elementMenu->plug(contextPopup);
  pageSizeMenu->plug(contextPopup);
  orientationMenu->plug(contextPopup);
  contextPopup->popup(e->globalPos());
  e->accept();
  kdDebug() << "Leaving LayoutTemplatePage::contextMenuEvent" << endl;
}

void LayoutTemplatePage::slotElementPageNumber()
{
}

void LayoutTemplatePage::slotElementTitle()
{
}

void LayoutTemplatePage::slotElementFabric()
{
}

void LayoutTemplatePage::slotElementFabricColor()
{
}

void LayoutTemplatePage::slotElementPageGrid()
{
}

void LayoutTemplatePage::slotElementCopyright()
{
}

void LayoutTemplatePage::slotElementAuthor()
{
}

void LayoutTemplatePage::slotPagePortrait()
{
  setOrientation(KPrinter::Portrait);
}

void LayoutTemplatePage::slotPageLandscape()
{
  setOrientation(KPrinter::Landscape);
}

void LayoutTemplatePage::slotPageA0()
{
  setPageSize(KPrinter::A0);
}

void LayoutTemplatePage::slotPageA1()
{
  setPageSize(KPrinter::A1);
}

void LayoutTemplatePage::slotPageA2()
{
  setPageSize(KPrinter::A2);
}

void LayoutTemplatePage::slotPageA3()
{
  setPageSize(KPrinter::A3);
}

void LayoutTemplatePage::slotPageA4()
{
  setPageSize(KPrinter::A4);
}

void LayoutTemplatePage::slotPageA5()
{
  setPageSize(KPrinter::A5);
}

void LayoutTemplatePage::slotPageA6()
{
  setPageSize(KPrinter::A6);
}

void LayoutTemplatePage::slotPageA7()
{
  setPageSize(KPrinter::A7);
}

void LayoutTemplatePage::slotPageA8()
{
  setPageSize(KPrinter::A8);
}

void LayoutTemplatePage::slotPageA9()
{
  setPageSize(KPrinter::A9);
}

void LayoutTemplatePage::slotPageB0()
{
  setPageSize(KPrinter::B0);
}

void LayoutTemplatePage::slotPageB1()
{
  setPageSize(KPrinter::B1);
}

void LayoutTemplatePage::slotPageB2()
{
  setPageSize(KPrinter::B2);
}

void LayoutTemplatePage::slotPageB3()
{
  setPageSize(KPrinter::B3);
}

void LayoutTemplatePage::slotPageB4()
{
  setPageSize(KPrinter::B4);
}

void LayoutTemplatePage::slotPageB5()
{
  setPageSize(KPrinter::B5);
}

void LayoutTemplatePage::slotPageB6()
{
  setPageSize(KPrinter::B6);
}

void LayoutTemplatePage::slotPageB7()
{
  setPageSize(KPrinter::B7);
}

void LayoutTemplatePage::slotPageB8()
{
  setPageSize(KPrinter::B8);
}

void LayoutTemplatePage::slotPageB9()
{
  setPageSize(KPrinter::B9);
}

void LayoutTemplatePage::slotPageB10()
{
  setPageSize(KPrinter::B10);
}

void LayoutTemplatePage::slotPageComm10E()
{
  setPageSize(KPrinter::Comm10E);
}

void LayoutTemplatePage::slotPageDLE()
{
  setPageSize(KPrinter::DLE);
}

void LayoutTemplatePage::slotPageExecutive()
{
  setPageSize(KPrinter::Executive);
}

void LayoutTemplatePage::slotPageFolio()
{
  setPageSize(KPrinter::Folio);
}

void LayoutTemplatePage::slotPageLedger()
{
  setPageSize(KPrinter::Ledger);
}

void LayoutTemplatePage::slotPageLegal()
{
  setPageSize(KPrinter::Legal);
}

void LayoutTemplatePage::slotPageLetter()
{
  setPageSize(KPrinter::Letter);
}

void LayoutTemplatePage::slotPageTabloid()
{
  setPageSize(KPrinter::Tabloid);
}

void LayoutTemplatePage::updatePageDisplay()
{
  kdDebug() << "Entering LayoutTemplatePage::updatePageDisplay" << endl;
  resize(width(), height());
  kdDebug() << "parentWidget()->width() = " << parentWidget()->width() << endl;
  kdDebug() << "parentWidget()->height() = " << parentWidget()->height() << endl;
  kdDebug() << "width() = " << width() << endl;
  kdDebug() << "height() = " << height() << endl;
  int parentWidth = parentWidget()->width();
  int parentHeight = parentWidget()->height();
  int xpos = (parentWidth > width())?(parentWidth-width())/2:0;
  int ypos = (parentHeight > height())?(parentHeight-height())/2:0;
  move(xpos, ypos);
  kdDebug() << "Leaving LayoutTemplatePage::updatePageDisplay" << endl;
}

void LayoutTemplatePage::drawGrid(QPainter *p)
{
  int w = width();
  int h = height();
  int horizontalMidPoint = w/2;
  int verticalMidPoint = h/2;
  m_gridXStart = horizontalMidPoint%m_gridX;
  m_gridYStart = verticalMidPoint%m_gridY;
  for (int x = m_gridXStart ; x < w ; x += m_gridX)
  {
    for (int y = m_gridYStart ; y < h ; y += m_gridY)
    {
      p->drawPoint(x, y);
    }
  }
}

LayoutTemplateDialog::LayoutTemplateDialog(QWidget *parent)
  : LayoutTemplateDlg(parent)
{
  kdDebug() << "Entering LayoutTemplateDialog::LayoutTemplateDialog" << endl;
  QStringList pageSizeStringList;
  for (int pageSize = 0 ; pageSize < NUMBERPAGESIZES ; pageSize++)
  {
    pageSizeStringList.append(QString(pageSizeList[pageSize].name));
  }
  Template_PageSize->insertStringList(pageSizeStringList);

  connect(Template_PrevPage, SIGNAL(clicked()), this, SLOT(slotPrevPage()));
  connect(Template_NextPage, SIGNAL(clicked()), this, SLOT(slotNextPage()));
  connect(Template_InsertPage, SIGNAL(clicked()), this, SLOT(slotInsertPage()));
  connect(Template_AppendPage, SIGNAL(clicked()), this, SLOT(slotAppendPage()));
  connect(Template_RotatePage, SIGNAL(clicked()), this, SLOT(slotRotatePage()));
  connect(Template_DeletePage, SIGNAL(clicked()), this, SLOT(slotDeletePage()));
  connect(Template_PageSize, SIGNAL(activated(int)), this, SLOT(slotSetPageSize(int)));

  m_pages.append(new LayoutTemplatePage(Template_PageView));
  m_currentPage = 1;
  Template_PageView->addChild(m_pages.at(m_currentPage-1));
  updatePageNumbers();
  updatePageOfPages();
  kdDebug() << "Leaving LayoutTemplateDialog::LayoutTemplateDialog" << endl;
}

LayoutTemplateDialog::~LayoutTemplateDialog()
{
  kdDebug() << "Entering LayoutTemplateDialog::~LayoutTemplateDialog" << endl;
  kdDebug() << "Leaving LayoutTemplateDialog::~LayoutTemplateDialog" << endl;
}

void LayoutTemplateDialog::accept()
{
  kdDebug() << "Entering LayoutTemplateDialog::accept" << endl;
  // call the base QDialog::accept function to close the dialog correctly
  QDialog::accept();
  kdDebug() << "Leaving LayoutTemplateDialog::accept" << endl;
}

void LayoutTemplateDialog::slotActivatedPageSize(int index)
{
  kdDebug() << "Entering LayoutTemplateDialog::slotActivatedPageSize" << endl;
  m_pages.at(m_currentPage-1)->setPageSize(index);
  kdDebug() << "Leaving LayoutTemplateDialog::slotActivatedPageSize" << endl;
}

void LayoutTemplateDialog::slotNextPage()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotNextPage" << endl;
  Template_PageView->removeChild(m_pages.at(m_currentPage-1));
  m_currentPage++;
  Template_PageView->addChild(m_pages.at(m_currentPage-1));
  updateButtons();
  updatePageOfPages();
  kdDebug() << "Leaving LayoutTemplateDialog::slotNextPage" << endl;
}

void LayoutTemplateDialog::slotPrevPage()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotPrevPage" << endl;
  Template_PageView->removeChild(m_pages.at(m_currentPage-1));
  m_currentPage--;
  Template_PageView->addChild(m_pages.at(m_currentPage-1));
  updateButtons();
  updatePageOfPages();
  kdDebug() << "Leaving LayoutTemplateDialog::slotPrevPage" << endl;
}

void LayoutTemplateDialog::slotRotatePage()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotRotatePage" << endl;
  LayoutTemplatePage *page = m_pages.at(m_currentPage-1);
  page->setOrientation((page->orientation() == KPrinter::Portrait)?KPrinter::Landscape:KPrinter::Portrait);
  kdDebug() << "Leaving LayoutTemplateDialog::slotRotatePage" << endl;
}

void LayoutTemplateDialog::slotInsertPage()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotInsertPage" << endl;
  m_pages.insert(m_currentPage-1, new LayoutTemplatePage(Template_PageView));
  updatePageNumbers();
  updateButtons();
  updatePageOfPages();
  kdDebug() << "Leaving LayoutTemplateDialog::slotInsertPage" << endl;
}

void LayoutTemplateDialog::slotAppendPage()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotAppendPage" << endl;
  m_pages.append(new LayoutTemplatePage(Template_PageView));
  m_currentPage = m_pages.count();
  updatePageNumbers();
  updateButtons();
  updatePageOfPages();
  kdDebug() << "Leaving LayoutTemplateDialog::slotAppendPage" << endl;
}

void LayoutTemplateDialog::slotDeletePage()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotDeletePage" << endl;
  LayoutTemplatePage *page = m_pages.take(m_currentPage-1);
  delete page;
  m_currentPage = (m_currentPage > m_pages.count())?m_pages.count():m_currentPage;
  updatePageNumbers();
  updateButtons();
  updatePageOfPages();
  kdDebug() << "Leaving LayoutTemplateDialog::slotDeletePage" << endl;
}

void LayoutTemplateDialog::slotSetPageSize(int i)
{
  kdDebug() << "Entering LayoutTemplateDialog::slotSetPageSize" << endl;
  m_pages.at(m_currentPage-1)->setPageSize(i);
  kdDebug() << "Leaving LayoutTemplateDialog::slotSetPageSize" << endl;
}

void LayoutTemplateDialog::slotContextHelp()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotContextHelp" << endl;
  kapp->invokeHelp("LayoutTemplateDialog");
  kdDebug() << "Leaving LayoutTemplateDialog::slotContextHelp" << endl;
}

void LayoutTemplateDialog::slotZoomIn()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotZoomIn" << endl;
  kdDebug() << "Leaving LayoutTemplateDialog::slotZoomIn" << endl;
}

void LayoutTemplateDialog::slotZoomOut()
{
  kdDebug() << "Entering LayoutTemplateDialog::slotZoomOut" << endl;
  kdDebug() << "Leaving LayoutTemplateDialog::slotZoomOut" << endl;
}

void LayoutTemplateDialog::updatePageNumbers()
{
  kdDebug() << "Entering LayoutTemplateDialog::updatePageNumbers" << endl;
  for (int i = 0 ; i < m_pages.count() ; i++)
  {
    m_pages.at(i)->setPageNumber(i);
  }
  kdDebug() << "Leaving LayoutTemplateDialog::updatePageNumbers" << endl;
}

void LayoutTemplateDialog::updatePageOfPages()
{
  kdDebug() << "Entering LayoutTemplateDialog::updatePageOfPages" << endl;
  QString pageOfPages = QString("Page %1 of %2").arg(m_currentPage).arg(m_pages.count());
  Template_PageOfPages->setText(pageOfPages);
  kdDebug() << "Leaving LayoutTemplateDialog::updatePageOfPages" << endl;
}

void LayoutTemplateDialog::updateButtons()
{
  kdDebug() << "Entering LayoutTemplateDialog::updateButtons" << endl;
  Template_NextPage->setEnabled(m_currentPage < m_pages.count());
  Template_PrevPage->setEnabled(m_currentPage > 1);
  Template_DeletePage->setEnabled(m_pages.count() > 1);
  kdDebug() << "Leaving LayoutTemplateDialog::updateButtons" << endl;
}
