/***************************************************************************
 *   Copyright (C) 2009 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qtimer.h>
#include <kdebug.h>
#include "librarylistview.h"
#include "kxstitchdragobject.h"
#include "patternlistviewitem.h"

#define AUTO_OPEN_DELAY 1

LibraryListView::LibraryListView(QWidget *parent, QString name)
  : KListView(parent, name),
    m_dropItem(0),
    m_currentItem(0)
{
  setAcceptDrops(true);
  m_openBranchTimer = new QTimer(this);
  connect(m_openBranchTimer, SIGNAL(timeout()), this, SLOT(slotOpenBranch()));
}

void LibraryListView::contentsDragEnterEvent(QDragEnterEvent *event)
{
  if (acceptDrag(event))
  {
    event->accept();
    m_currentItem = selectedItem();
    QListViewItem *item = itemAt(contentsToViewport(event->pos()));
    if (item)
    {
      m_dropItem = item;
      m_openBranchTimer->start(AUTO_OPEN_DELAY);
    }
    else
      m_dropItem = 0;
  }
  else
    event->ignore();
}

void LibraryListView::contentsDragMoveEvent(QDragMoveEvent *event)
{
  if (acceptDrag(event))
  {
    event->accept();
    QListViewItem *item = itemAt(contentsToViewport(event->pos()));
    if (item)
    {
      setSelected(item, true);
      if (item != m_dropItem)
      {
        m_openBranchTimer->stop();
        m_dropItem = item;
        m_openBranchTimer->start(AUTO_OPEN_DELAY); // TODO change this to a configuration object
      }
    }
    else
    {
      m_openBranchTimer->stop();
      m_dropItem = 0;
    }
  }
  else
  {
    m_openBranchTimer->stop();
    m_dropItem = 0;
  }
}

void LibraryListView::contentsDragLeaveEvent(QDragLeaveEvent *)
{
  if (m_currentItem)
  {
    setSelected(m_currentItem, true);
    ensureItemVisible(m_currentItem);
  }
  else
    setSelected(m_dropItem, false);
  m_currentItem = 0;
  m_dropItem = 0;
}

void LibraryListView::contentsDropEvent(QDropEvent *event)
{
  m_openBranchTimer->stop();
  if (m_dropItem && acceptDrag(event))
  {
    QWidget *source = event->source(); // may be 0 if dragging from another app
    if (source && (strcmp( source->name(), "PatternPreviews") == 0) && m_dropItem == m_currentItem)
    {
      // ignore this drag as it is a within the same library
    }
    else
    {
      static_cast<PatternListViewItem*>(m_dropItem)->addPattern(new KXStitchDragObject(event->encodedData("application/kxstitch")));
    }
  }
  m_dropItem = 0;
  if (m_currentItem)
    setSelected(m_currentItem, true);
}

bool LibraryListView::acceptDrag(QDropEvent *event) const
{
  return event->provides("application/kxstitch");
}

void LibraryListView::slotOpenBranch()
{
  m_openBranchTimer->stop();
  if (m_dropItem && !m_dropItem->isOpen())
  {
    m_dropItem->setOpen(true);
  }
}
