/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

void PatternLibraryDlg::init()
{
  // Initialise the dialog, called from the constructor
  PatternList->addColumn(tr2i18n( "Libraries"));
  PatternList->header()->setClickEnabled(FALSE, PatternList->header()->count()-1);
  PatternList->header()->setResizeEnabled(FALSE, PatternList->header()->count()-1);
  PatternList->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)5, (QSizePolicy::SizeType)7, 0, 0, PatternList->sizePolicy().hasHeightForWidth()));
  PatternList->setRootIsDecorated(TRUE);
  PatternList->setFullWidth(TRUE);

  connect(PatternList, SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)), this, SLOT(PatternList_contextMenuRequested(QListViewItem *, const QPoint &, int)));
  connect(PatternList, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(PatternList_selectionChanged(QListViewItem *)));

  PatternList->setSorting(0, true);
  PatternList->clear();
  QStringList libraries = kapp->dirs()->findDirs("appdata", "library");
  for (QStringList::Iterator it = libraries.begin() ; it != libraries.end() ; ++it)
    recurseLibrary(0, *it);
  if (PatternList->firstChild())
  {
    PatternList->setSelected(PatternList->firstChild(), true);
    PatternList_selectionChanged(static_cast<QListViewItem *>(PatternList->firstChild()));
  }
}

void PatternLibraryDlg::destroy()
{
  // Destroy the dialog, called from the destructor
}

void PatternLibraryDlg::recurseLibrary(PatternListViewItem *listItem, QString &path)
{
  // Some of the paths will be global and some local to the user.  This also means some will be writable and some not.
  // Various root paths are searched and the hierarchy is merged into a single tree with multiple paths.
  PatternListViewItem *item = 0;
  QDir dir(path);
  const QFileInfoList *dirEntries = dir.entryInfoList();
  QPtrListIterator<QFileInfo> files(*dirEntries);

  for (QFileInfo *file = 0 ; (file = files.current()) ; ++files)
  {
    if (file->isDir() && file->fileName() != "." && file->fileName() != "..") // We are only interested in the folder hierarchy
    {
      QString subPath = QString("%1%2/").arg(path).arg(file->fileName());
      if (listItem == 0) // there is no parent item so add to root as first level node
        item = static_cast<PatternListViewItem *>(PatternList->findItem(file->fileName(), 0, Qt::ExactMatch)); // see if there is already a root node with this name
      else
      {
        item = static_cast<PatternListViewItem *>(listItem->firstChild()); // Iterate through listItem children
        while (item)
        {
          if (item->text(0) == file->fileName()) // see if there is already a branch node with this name
            break;
          else
            item = static_cast<PatternListViewItem *>(item->nextSibling());
        }
      }
      // item will either point to a tree node with this name or will be null
      if (item == 0)
      {
        // create a new item either as a root node or as a child of another node
        if (listItem)
          item = new PatternListViewItem(listItem, file->fileName());
        else
          item = new PatternListViewItem(PatternList, file->fileName());
      }
      item->addPath(subPath);
      recurseLibrary(item, subPath);
    }
  }
}

void PatternLibraryDlg::PatternList_contextMenuRequested(QListViewItem *item, const QPoint &p, int)
{
  m_contextMenu.clear();
  m_contextMenu.insertItem(i18n("New Catagory"), this, SLOT(slotNewCatagory()));
  if (item)
  {
    m_contextMenu.insertItem(i18n("Add to Export List"), this, SLOT(slotAddLibraryToExportList()));
    m_contextMenu.insertItem(i18n("Properties..."), this, SLOT(slotPatternLibraryProperties()));
    if (kapp->clipboard()->data()->provides("application/kxstitch"))
    {
      m_contextMenu.insertItem(i18n("Paste"), this, SLOT(slotPasteFromClipboard()));
    }
  }
  m_contextMenu.popup(p);
}

void PatternLibraryDlg::PatternPreviews_contextMenuRequested(QIconViewItem *item, const QPoint &p)
{
  m_contextMenu.clear();
  if (item)
  {
    m_contextMenu.insertItem(i18n("Properties..."), this, SLOT(slotProperties()));
    m_contextMenu.insertItem(i18n("Add to Export List"), this, SLOT(slotAddPatternToExportList()));
    m_contextMenu.insertItem(i18n("Copy"), this, SLOT(slotCopyToClipboard()));
    m_contextMenu.insertItem(i18n("Delete"), this, SLOT(slotDeletePattern()));
    m_contextMenu.popup(p);
  }
  else
  {
    if (kapp->clipboard()->data()->provides("application/kxstitch") && PatternList->selectedItem())
    {
      m_contextMenu.insertItem(i18n("Paste"), this, SLOT(slotPasteFromClipboard()));
      m_contextMenu.popup(p);
    }
  }
}

void PatternLibraryDlg::ImportButton_clicked()
{
  KURL url = KFileDialog::getOpenURL(QString("::%1").arg(KGlobalSettings::documentPath()), i18n("*|All files"), this, i18n("Import..."));
  if(!url.isEmpty())
  {
    PatternFile importFile(url.path());
    for (LibraryPattern *pattern = importFile.first() ; pattern ; pattern = importFile.next())
    {
      currentLibrary()->addPattern(pattern);
    }
    PatternList_selectionChanged(currentLibrary());
  }
}

void PatternLibraryDlg::ExportButton_clicked()
{
  bool cancel = false;
  KURL url;

  while (1)
  {
    QFile exportFile;
    url = KFileDialog::getSaveURL(QString("::%1 ").arg( KGlobalSettings::documentPath()), i18n("*|All files"), this, i18n("Export as..."));
    if (cancel = url.isEmpty()) // cancel was pressed
      break;
    else
    {
      exportFile.setName(url.path());
      if (!exportFile.exists())
        break;
      else
        if (KMessageBox::warningYesNo(0, i18n("The file exists.\nDo you want to overwrite it.")) == KMessageBox::Yes)
          break;
    }
  }

  if (!cancel)
  {
    PatternFile exportFile(url.path());
    for (LibraryPattern *pattern = m_exportPatterns.first() ; pattern ; pattern = m_exportPatterns.next())
    {
      exportFile.addPattern(pattern);
    }
    m_exportPatterns.clear();
    ExportButton->setEnabled(m_exportPatterns.count());
  }
}

void PatternLibraryDlg::PatternList_selectionChanged(QListViewItem *item)
{
  PatternPreviews->clear();
  // iterate the paths associated with the item
  for (LibraryPattern *libraryPattern = static_cast<PatternListViewItem *>(item)->first() ; libraryPattern ; libraryPattern = static_cast<PatternListViewItem *>(item)->next())
  {
    PreviewIconViewItem *item = new PreviewIconViewItem(PatternPreviews, libraryPattern);
    libraryPattern->setIconViewItem(item);
  }
}

void PatternLibraryDlg::slotNewCatagory()
{
  QString catagory;
  PatternListViewItem *listItem = static_cast<PatternListViewItem *>(PatternList->selectedItem());
  bool ok = false;

  while (!ok)
  {
    QListViewItem *item;
    catagory = KLineEditDlg::getText(i18n("Catagory"), "", &ok, this);
    if (!ok)
      // user pressed cancel
      break;
    if (listItem)
    {
      for (item = listItem->firstChild() ; item ; item = item->nextSibling())
        if (item->text(0) == catagory)
          break;
    }
    else
      item = PatternList->findItem(catagory, 0, Qt::ExactMatch);

    if (item)
    {
      KMessageBox::sorry(this, i18n("This catagory already exists."), i18n("Catagory Exists"));
      ok = false;
    }
  }

  if (ok)
  {
    // create a new PatternViewListItem as a child of the currently selected list item or as a new root node if no list item is currently selected.
    // note: there is a difference between the currently selected list item and current list item
    PatternListViewItem *item;
    if (listItem)
      item = new PatternListViewItem(listItem, catagory);
    else
      item = new PatternListViewItem(PatternList, catagory);

    // create a path from the listItem path to a new sub path
    QString path;
    QFileInfo fileInfo;

    if (listItem)
    {
      fileInfo.setFile(listItem->path());
      if (!fileInfo.isWritable())
      {
        path.replace(0, path.find("library"), "");
        path = kapp->dirs()->saveLocation("appdata", path);
        fileInfo.setFile(path);
      }
    }
    else
    {
      path = kapp->dirs()->saveLocation("appdata", "library");
      fileInfo.setFile(path);
    }

    if (fileInfo.dir().mkdir(catagory))
    {
      path += catagory;
      path += "/";
      item->addPath(path);
    }

    PatternList->ensureItemVisible(item);
    PatternList->setCurrentItem(item);
  }
}

void PatternLibraryDlg::slotAddLibraryToExportList()
{
  PatternListViewItem *library = currentLibrary();
  for (LibraryPattern *pattern = library->first() ; pattern ; pattern = library->next())
    m_exportPatterns.append(pattern);
  ExportButton->setEnabled(m_exportPatterns.count() > 0);
}

void PatternLibraryDlg::slotProperties()
{
  LibraryPatternPropertiesDlg *dlg = new LibraryPatternPropertiesDlg();
  LibraryPattern *pattern = currentIconPattern();
  KXStitchDragObject mimeSource(pattern->data());
  mimeSource.setScale(Configuration::library_IconViewScale());
  dlg->DetailPreview->setPixmap(mimeSource.encodedData("image/png"));
  dlg->FlossScheme->setText(pattern->scheme());
  dlg->PatternWidth->setText(QString().setNum(pattern->width()));
  dlg->PatternHeight->setText(QString().setNum(pattern->height()));
  dlg->PatternBaseline->setValue(pattern->baseline());
  dlg->PatternCharacter->setKeyModifiers(pattern->key(), pattern->modifiers());
  if (dlg->exec() == QDialog::Accepted)
  {
    pattern->setBaseline(dlg->PatternBaseline->value());
    pattern->setKeyModifiers(dlg->PatternCharacter->key(), dlg->PatternCharacter->modifiers());
  }
  delete dlg;
}

void PatternLibraryDlg::slotAddPatternToExportList()
{
  LibraryPattern *pattern = currentIconPattern();
  if (pattern)
    m_exportPatterns.append(pattern);
  ExportButton->setEnabled(m_exportPatterns.count() > 0);
}

void PatternLibraryDlg::slotCopyToClipboard()
{
  kapp->clipboard()->setData(new KXStitchDragObject(currentIconPattern()->data()));
}

void PatternLibraryDlg::slotPasteFromClipboard()
{
  PatternListViewItem *item = static_cast<PatternListViewItem *>(PatternList->selectedItem());
  item->addPattern(static_cast<KXStitchDragObject *>(kapp->clipboard()->data()));
  PatternList_selectionChanged(static_cast<QListViewItem *>(item));
}

void PatternLibraryDlg::slotDeletePattern()
{
  if (KMessageBox::warningYesNo(0, i18n("Delete this pattern.")) == KMessageBox::Yes)
  {
    PreviewIconViewItem *iconItem = static_cast<PreviewIconViewItem *>(PatternPreviews->currentItem());
    static_cast<PatternListViewItem *>(PatternList->currentItem())->deletePattern(iconItem);
    delete iconItem; // also removes it from the icon view
  }
}

LibraryPattern *PatternLibraryDlg::currentIconPattern()
{
  return static_cast<PreviewIconViewItem *>(PatternPreviews->currentItem())->libraryPattern();
}

PatternListViewItem *PatternLibraryDlg::currentLibrary()
{
  return static_cast<PatternListViewItem *>(PatternList->currentItem());
}

void PatternLibraryDlg::setScaleSize(int scaleSize)
{
  PatternPreviews->setScaleSize(scaleSize);
}

void PatternLibraryDlg::PatternList_dropped(QDropEvent *, QListViewItem *)
{
  kdDebug() << "Entering " << k_funcinfo << endl;
}

void PatternLibraryDlg::slotPatternLibraryProperties()
{
  PatternLibraryPropertiesDialog *dialog = new PatternLibraryPropertiesDialog(this);
  PatternListViewItem *item = static_cast<PatternListViewItem *>(PatternList->currentItem());
  dialog->setCatagory(item->text(0));
  QStringList paths = item->paths();
  for (QStringList::iterator it = paths.begin() ; it != paths.end() ; ++it)
    dialog->addPath(*it);
  dialog->exec();
  delete dialog;
}
