/***************************************************************************
 *   Copyright (C) 2003 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qcheckbox.h>
#include <qspinbox.h>
#include <kcombobox.h>
#include <kprinter.h>
#include <klocale.h>
#include "configuration.h"
#include "printdlgpage.h"
#include "printdlg.h"

PrintDlgPage::PrintDlgPage( KPrinter *prn, QSize cells, QWidget *parent, const char *name)
  : KPrintDialogPage(parent, name),
    m_printer(prn),
    m_gridSize(cells),
    m_dlg(new PrintDlgUI(this))
{
  setTitle(i18n("KXStitch Print Options"));
}

void PrintDlgPage::getOptions(QMap<QString, QString> &opts, bool)
{
  if (m_dlg->Print_SetDefault->isChecked())
  {
    Configuration::setPrint_FrontPage(m_dlg->Print_FrontPage->isChecked());
    Configuration::setPrint_Instructions(m_dlg->Print_Instructions->isChecked());
    Configuration::setPrint_Pattern(m_dlg->Print_Pattern->isChecked());
    Configuration::setPrint_SquaresInch(m_dlg->Print_SquaresInch->value()); // TODO modify to use users preferred units
    Configuration::setPrint_SpillOver(m_dlg->Print_SpillOver->value());
    Configuration::setPrint_FitSingle(m_dlg->Print_FitSingle->isChecked());
    Configuration::setPrint_GridLines(m_dlg->Print_GridLines->isChecked());
    Configuration::setPrint_SelectedArea(m_dlg->Print_SelectedArea->isChecked());
    Configuration::setPrint_StitchesAs(m_dlg->Print_StitchesAs->currentItem());
    Configuration::setPrint_BackstitchesAs(m_dlg->Print_BackstitchesAs->currentItem());
    Configuration::setPrint_ColorKey(m_dlg->Print_ColorKey->isChecked());
    Configuration::setPrint_FlossUsage(m_dlg->Print_FlossUsage->isChecked());
    Configuration::setPrint_StitchCount(m_dlg->Print_StitchCount->isChecked());
    Configuration::setPrint_CombineFractionals(m_dlg->Print_CombineFractionals->isChecked());
    Configuration::setPrint_SeperateBackstitching(m_dlg->Print_SeperateBackstitching->isChecked());
    Configuration::setPrint_StitchTotals(m_dlg->Print_StitchTotals->isChecked());
    Configuration::setPrint_StitchSummary(m_dlg->Print_StitchSummary->isChecked());
  }
  opts["kde-kxstitch-frontpage"] = (m_dlg->Print_FrontPage->isChecked()?"true":"false");
  opts["kde-kxstitch-instructions"] = (m_dlg->Print_Instructions->isChecked()?"true":"false");
  opts["kde-kxstitch-pattern"] = (m_dlg->Print_Pattern->isChecked()?"true":"false");
  opts["kde-kxstitch-squaresinch"] = QString("%1").arg(m_dlg->Print_SquaresInch->value());
  opts["kde-kxstitch-spillover"] = QString("%1").arg( m_dlg->Print_SpillOver->value());
  opts["kde-kxstitch-fitsingle"] = (m_dlg->Print_FitSingle->isChecked()?"true":"false");
  opts["kde-kxstitch-gridlines"] = (m_dlg->Print_GridLines->isChecked()?"true":"false");
  opts["kde-kxstitch-selectedarea"] = (m_dlg->Print_SelectedArea->isChecked()?"true":"false");
  opts["kde-kxstitch-stitches"] = m_dlg->Print_StitchesAs->currentText();
  opts["kde-kxstitch-backstitches"] = m_dlg->Print_BackstitchesAs->currentText();
  opts["kde-kxstitch-colorkey"] = (m_dlg->Print_ColorKey->isChecked()?"true":"false");
  opts["kde-kxstitch-flossusage"] = (m_dlg->Print_FlossUsage->isChecked()?"true":"false");
  opts["kde-kxstitch-stitchcount"] = (m_dlg->Print_StitchCount->isChecked()?"true":"false");
  opts["kde-kxstitch-combinefractionals"] = (m_dlg->Print_CombineFractionals->isChecked()?"true":"false");
  opts["kde-kxstitch-seperatebackstitching"] = (m_dlg->Print_SeperateBackstitching->isChecked()?"true":"false");
  opts["kde-kxstitch-stitchtotals"] = (m_dlg->Print_StitchTotals->isChecked()?"true":"false");
  opts["kde-kxstitch-stitchsummary"] = (m_dlg->Print_StitchSummary->isChecked()?"true":"false");
}

void PrintDlgPage::setOptions(const QMap<QString, QString> &opts)
{
  m_dlg->Print_FrontPage->setChecked(opts.contains("kde-kxstitch-frontpage")?((opts["kde-kxstitch-frontpage"] == "true")?true:false):Configuration::print_FrontPage());
  m_dlg->Print_Instructions->setChecked(opts.contains("kde-kxstitch-instructions")?((opts["kde-kxstitch-instructions"] == "true")?true:false):Configuration::print_Instructions());
  m_dlg->Print_Pattern->setChecked(opts.contains("kde-kxstitch-pattern")?((opts["kde-kxstitch-pattern"] == "true")?true:false):Configuration::print_Pattern());
  m_dlg->Print_SquaresInch->setValue(opts.contains("kde-kxstitch-squaresinch")?opts["kde-kxstitch-squaresinch"].toInt():Configuration::print_SquaresInch());
  m_dlg->Print_SpillOver->setValue(opts.contains("kde-kxstitch-spillover")?opts["kde-kxstitch-spillover"].toInt():Configuration::print_SpillOver());
  m_dlg->Print_FitSingle->setChecked(opts.contains("kde-kxstitch-fitsingle")?((opts["kde-kxstitch-fitsingle"] == "true")?true:false):Configuration::print_FitSingle());
  m_dlg->Print_GridLines->setChecked(opts.contains("kde-kxstitch-gridlines")?((opts["kde-kxstitch-gridlines"] == "true")?true:false):Configuration::print_GridLines());
  m_dlg->Print_SelectedArea->setChecked( opts.contains("kde-kxstitch-selectedarea")?((opts["kde-kxstitch-selectedarea"] == "true")?true:false):Configuration::print_SelectedArea());
  if (opts.contains("kde-kxstitch-stitches"))
    m_dlg->Print_StitchesAs->setCurrentItem(opts["kde-kxstitch-stitches"]);
  else
    m_dlg->Print_StitchesAs->setCurrentItem(Configuration::print_StitchesAs());
  if (opts.contains("kde-kxstitch-backstitches"))
    m_dlg->Print_BackstitchesAs->setCurrentItem(opts["kde-kxstitch-backstitches"]);
  else
    m_dlg->Print_BackstitchesAs->setCurrentItem(Configuration::print_BackstitchesAs());
  m_dlg->Print_ColorKey->setChecked(opts.contains("kde-kxstitch-colorkey")?((opts["kde-kxstitch-colorkey"] == "true")?true:false):Configuration::print_ColorKey());
  m_dlg->Print_FlossUsage->setChecked(opts.contains("kde-kxstitch-flossusage")?((opts["kde-kxstitch-flossusage"] == "true")?true:false):Configuration::print_FlossUsage());
  m_dlg->Print_StitchCount->setChecked(opts.contains("kde-kxstitch-stitchcount")?((opts["kde-kxstitch-stitchcount"] == "true")?true:false):Configuration::print_StitchCount());
  m_dlg->Print_CombineFractionals->setChecked(opts.contains("kde-kxstitch-combinefractionals")?((opts["kde-kxstitch-combinefractionals"] == "true")?true:false):Configuration::print_CombineFractionals());
  m_dlg->Print_SeperateBackstitching->setChecked(opts.contains("kde-kxstitch-seperatebackstitching")?((opts["kde-kxstitch-seperatebackstitching"] == "true")?true:false):Configuration::print_SeperateBackstitching());
  m_dlg->Print_StitchTotals->setChecked(opts.contains("kde-kxstitch-stitchtotals")?((opts["kde-kxstitch-stitchtotals"] == "true")?true:false):Configuration::print_StitchTotals());
  m_dlg->Print_StitchSummary->setChecked(opts.contains("kde-kxstitch-stitchsummary")?((opts["kde-kxstitch-stitchsummary"] == "true")?true:false):Configuration::print_StitchSummary());
}

bool PrintDlgPage::isValid(QString &)
{
/*  TODO check values of the dialog box are sane.
    For now just return TRUE.
  */
  return true;
}

QSize PrintDlgPage::minimumSizeHint() const
{
  return m_dlg->size();
}
